<?php

class Calculator {

    const WEEKS_PER_YEAR = 52;
    const MONTHS_PER_YEAR = 12;
    const TOTAL_YEARS = 3;

    #ASSUMPTIONS
    const COST_PER_EMPLOYEE = 30;
    const PERCENT_CUSTOMERS_LOST_WAIT_TIME = 0.33;
    const WORKLOAD_IMPROVEMENT_WITH_WEAVE = 0.5;

    private $totalYears = self::TOTAL_YEARS;

    #Workforce
    private $totalNumberLocations = 10;
    private $teamsPerLocation = 1;
    private $employeesPerTeam = 5;
    private $costPerEmployee = self::COST_PER_EMPLOYEE;

    #Workload
    private $typicalOrdersPerHourLocation = 5.0;
    private $averageProfitPerOrder = 50.0;
    private $durationBetweenRepeatCustomerVisits = 36;
    private $percentCustomersLostWaitTime = self::PERCENT_CUSTOMERS_LOST_WAIT_TIME; #33%

    #Matching Indicators

    #improvement Fixo em 50%
    private $overStaffedCurrent = 7;
    private $overStaffedImprovement = self::WORKLOAD_IMPROVEMENT_WITH_WEAVE;

    #improvement Fixo em 50%
    private $underStaffedCurrent = 6;
    private $underStaffedImprovement = self::WORKLOAD_IMPROVEMENT_WITH_WEAVE;

    #improvement Fixo em 50%
    private $overTimeCurrent = 20;
    private $overTimeImprovement = self::WORKLOAD_IMPROVEMENT_WITH_WEAVE;

    #Weave Workforce
    private $monthlySubscriptionCostPerTeam = -150.0;
    private $totalSubscriptionCost = -1500.0;
    private $evaluationAndSetup = 500;

    public static function formatNumber($value, $withSymbol = true) {
        $value = floatval($value);
        return ($withSymbol ? '$ ' : '') . number_format($value, 1, '.', ',');
    }

    public static function formatNegativeNumber($value) {
        $value = floatval($value);
        return '$ (' . number_format(abs($value), 1, '.', ',') . ')';
    }

    public function getTotalYears()
    {
        return $this->totalYears;
    }

    public function getTotalNumberLocations()
    {
        return $this->totalNumberLocations;
    }

    public function getTeamsPerLocation()
    {
        return $this->teamsPerLocation;
    }

    public function getEmployeesPerTeam()
    {
        return $this->employeesPerTeam;
    }

    public function getCostPerEmployee()
    {
        return $this->costPerEmployee;
    }

    public function getTypicalOrdersPerHourLocation()
    {
        return $this->typicalOrdersPerHourLocation;
    }

    public function getAverageProfitPerOrder()
    {
        return $this->averageProfitPerOrder;
    }

    public function getDurationBetweenRepeatCustomerVisits()
    {
        return $this->durationBetweenRepeatCustomerVisits;
    }

    public function getPercentCustomersLostWaitTime()
    {
        return $this->percentCustomersLostWaitTime;
    }

    public function getOverStaffedCurrent()
    {
        return $this->overStaffedCurrent;
    }

    public function getOverStaffedImprovement()
    {
        return $this->overStaffedImprovement;
    }

    public function getUnderStaffedCurrent()
    {
        return $this->underStaffedCurrent;
    }

    public function getUnderStaffedImprovement()
    {
        return $this->underStaffedImprovement;
    }

    public function getOverTimeCurrent()
    {
        return $this->overTimeCurrent;
    }

    public function getOverTimeImprovement()
    {
        return $this->overTimeImprovement;
    }

    public function getMonthlySubscriptionCostPerTeam()
    {
        return $this->monthlySubscriptionCostPerTeam;
    }

    public function getTotalSubscriptionCost()
    {
        return $this->totalSubscriptionCost;
    }

    public function getEvaluationAndSetup()
    {
        return $this->evaluationAndSetup;
    }
    
    public function setTotalYears($totalYears)
    {
        $this->totalYears = $totalYears;
    }

    public function setTotalNumberLocations($totalNumberLocations)
    {
        $this->totalNumberLocations = $totalNumberLocations;
    }

    public function setTeamsPerLocation($teamsPerLocation)
    {
        $this->teamsPerLocation = $teamsPerLocation;
    }

    public function setEmployeesPerTeam($employeesPerTeam)
    {
        $this->employeesPerTeam = $employeesPerTeam;
    }

    public function setCostPerEmployee($costPerEmployee)
    {
        $this->costPerEmployee = $costPerEmployee;
    }

    public function setTypicalOrdersPerHourLocation($typicalOrdersPerHourLocation)
    {
        $this->typicalOrdersPerHourLocation = $typicalOrdersPerHourLocation;
    }

    public function setAverageProfitPerOrder($averageProfitPerOrder)
    {
        $this->averageProfitPerOrder = $averageProfitPerOrder;
    }

    public function setDurationBetweenRepeatCustomerVisits($durationBetweenRepeatCustomerVisits)
    {
        $this->durationBetweenRepeatCustomerVisits = $durationBetweenRepeatCustomerVisits;
    }

    public function setPercentCustomersLostWaitTime($percentCustomersLostWaitTime)
    {
        $this->percentCustomersLostWaitTime = $percentCustomersLostWaitTime;
    }

    public function setOverStaffedCurrent($overStaffedCurrent)
    {
        $this->overStaffedCurrent = $overStaffedCurrent;
    }

    public function setOverStaffedImprovement($overStaffedImprovement)
    {
        $this->overStaffedImprovement = $overStaffedImprovement;
    }

    public function setUnderStaffedCurrent($underStaffedCurrent)
    {
        $this->underStaffedCurrent = $underStaffedCurrent;
    }

    public function setUnderStaffedImprovement($underStaffedImprovement)
    {
        $this->underStaffedImprovement = $underStaffedImprovement;
    }

    public function setOverTimeCurrent($overTimeCurrent)
    {
        $this->overTimeCurrent = $overTimeCurrent;
    }

    public function setOverTimeImprovement($overTimeImprovement)
    {
        $this->overTimeImprovement = $overTimeImprovement;
    }

    public function setMonthlySubscriptionCostPerTeam($monthlySubscriptionCostPerTeam)
    {
        $this->monthlySubscriptionCostPerTeam = $monthlySubscriptionCostPerTeam;
    }

    public function setEvaluationAndSetup($evaluationAndSetup)
    {
        $this->evaluationAndSetup = $evaluationAndSetup;
    }

    public function calcTotalSubscriptionCost()
    {
        return $this->getTotalNumberLocations() * $this->getMonthlySubscriptionCostPerTeam();
    }

    #calculadora B41
    public function calcOverStaffingPerHour()
    {
        $result = $this->getOverStaffedCurrent() * self::WEEKS_PER_YEAR * $this->getEmployeesPerTeam();
            
        return $result * $this->getTeamsPerLocation() * $this->getTotalNumberLocations();
    }
    
    #calculadora B42
    public function calcUnderStaffingPerHour()
    {
        $result = $this->getUnderStaffedCurrent() * self::WEEKS_PER_YEAR * $this->getEmployeesPerTeam();
            
        return $result * $this->getTeamsPerLocation() * $this->getTotalNumberLocations();
    }
    
    #calculadora B43
    public function calcOverTimePerHour()
    {
        return $this->getOverTimeCurrent() * $this->getTotalNumberLocations() * self::WEEKS_PER_YEAR;
    }
    
    #calculadora C41
    public function calcOverStaffingCost()
    {
        return bcmul($this->getCostPerEmployee(), $this->calcOverStaffingPerHour(), 10);
    }
    
    #calculadora C42
    public function calcUnderStaffingCost()
    {
        return bcmul($this->getCostPerEmployee(), $this->calcUnderStaffingPerHour() * -1, 10);
    }
    
    #calculadora C43
    public function calcOverTimeCost()
    {
        $result = bcmul($this->getCostPerEmployee(), $this->calcOverTimePerHour(), 10);
        return bcmul($result, 0.5, 10);
    }
    
    #calculadora D41
    public function calcOverStaffingReduction()
    {
        return bcmul($this->calcOverStaffingCost(), $this->getOverStaffedImprovement(), 10);
    }
    
    #calculadora D42
    public function calcUnderStaffingReduction()
    {
        return bcmul($this->calcUnderStaffingCost(), $this->getUnderStaffedImprovement(), 10);
    }

    #calculadora D43
    public function calcOverTimeReduction()
    {
        return bcmul($this->calcOverTimeCost(), $this->getOverTimeImprovement(), 10);
    }

    public function calcValuesByWeeks()
    {
        $week = 0;
        $evaluationAndSetup = [];
        $subscription = [];
        $softwareExpense = [];
        $subtotalSoftwareExpense = [];
        $totalNetImpact = [];
        $recurrentVisitsLost = [];
        $lostMarginPerStoge = [];
        $lostMarginPerLocation = [];
        $customerRetentionProfits = [];
        $overStaffingReductionByWeek = [];
        $underStaffingReductionByWeek = [];
        $overTimeReductionByWeek = [];
        $cumulativeUnderStaffingLaborCostReduction = [];

        $evaluationAndSetup[$week] = $this->getEvaluationAndSetup();
        $customerRetentionProfits[$week] = 0;
        $subscription[$week] = 0;
        $subtotalSoftwareExpense[$week] = $evaluationAndSetup[$week] + $subscription[$week];

        $totalNetImpact[$week] = $evaluationAndSetup[$week] + $subscription[$week] + $subtotalSoftwareExpense[$week];

        $cumulativeUnderStaffingLaborCostReduction[$week] = 0;

        for($week = 1; $week <= self::WEEKS_PER_YEAR * $this->getTotalYears(); $week++) {
           
            $previousWeek = $week - 1;

            $evaluationAndSetup[$week] = 0;
            $subscription[$week] = $this->calcTotalSubscriptionCost() * (self::MONTHS_PER_YEAR / self::WEEKS_PER_YEAR);
            $subtotalSoftwareExpense[$week] = $evaluationAndSetup[$week] + $subscription[$week];

            $overStaffingReductionByWeek[$week] = $this->calcOverStaffingReduction() / self::WEEKS_PER_YEAR;
            $underStaffingReductionByWeek[$week] = $this->calcUnderStaffingReduction() / self::WEEKS_PER_YEAR;
            $overTimeReductionByWeek[$week] = $this->calcOverTimeReduction() / self::WEEKS_PER_YEAR;
            $subtotalLaborCost[$week] = $overStaffingReductionByWeek[$week] + $underStaffingReductionByWeek[$week] + $overTimeReductionByWeek[$week];


            $recurrentVisitsLost[$week] = floor($week / $this->getDurationBetweenRepeatCustomerVisits()) * 
                                    $this->getTypicalOrdersPerHourLocation() * 
                                    $this->getUnderStaffedCurrent() * 
                                    $this->getPercentCustomersLostWaitTime();
            
            $lostMarginPerStoge[$week] = $recurrentVisitsLost[$week] * $this->getAverageProfitPerOrder() * $this->getUnderStaffedImprovement();
            $lostMarginPerLocation[$week] = $lostMarginPerStoge[$week] * $this->getTotalNumberLocations();

            $customerRetentionProfits[$week] = $lostMarginPerLocation[$week] + $customerRetentionProfits[$previousWeek];

            $netBusinessImpact[$week] = $subtotalSoftwareExpense[$week] + 
                                        $lostMarginPerLocation[$week] +
                                        $subtotalLaborCost[$week];

            $softwareExpense[$week] = $subtotalSoftwareExpense[$previousWeek] + $subtotalSoftwareExpense[$week];
            $totalNetImpact[$week] = $totalNetImpact[$previousWeek] + $netBusinessImpact[$week];
            $cumulativeUnderStaffingLaborCostReduction[$week] = $cumulativeUnderStaffingLaborCostReduction[$previousWeek] + $underStaffingReductionByWeek[$week];
        }

        $result = [
            'evaluationAndSetup' => $evaluationAndSetup,
            'subscription' => $subscription,
            'subtotalSoftwareExpense' => $subtotalSoftwareExpense,
            'overStaffingReductionByWeek' => $overStaffingReductionByWeek,
            'underStaffingReductionByWeek' => $underStaffingReductionByWeek,
            'overTimeReductionByWeek' => $overTimeReductionByWeek,
            'subtotalLaborCost' => $subtotalLaborCost,
            'recurrentVisitsLost' => $recurrentVisitsLost,
            'lostMarginPerStoge' => $lostMarginPerStoge,
            'lostMarginPerLocation' => $lostMarginPerLocation,
            'customerRetentionProfits' => $customerRetentionProfits,
            'netBusinessImpact' => $netBusinessImpact,
            'softwareExpense' => $softwareExpense,
            'totalNetImpact' => $totalNetImpact,
            'cumulativeUnderStaffingLaborCostReduction' => $cumulativeUnderStaffingLaborCostReduction,
        ];

        return $result;
    }

    public function calcValuesByYear()
    {
        $year = 0;
        $customerRetentionProfits = [];
        
        $overStaffingRedution = [];
        $underStaffingRedution = [];
        $overTimeRedution = [];
        $laborCostImprovement = [];

        $totalSubscriptionCost = [];
        $totalCompany = [];

        $customerRetentionProfits[$year] = 0;
        $overStaffingRedution[$year] = 0;
        $underStaffingRedution[$year] = 0;
        $overTimeRedution[$year] = 0;
        $laborCostImprovement[$year] = 0;
        $totalSubscriptionCost[$year] = 0;
        $totalCompany[$year] = 0;

        $dataByMonths = $this->calcValuesByWeeks();

        for ($year=1; $year <= $this->getTotalYears() ; $year++) { 
            $previousYear = $year - 1;

            $customerRetentionProfits[$year] = $dataByMonths['customerRetentionProfits'][$year * self::WEEKS_PER_YEAR];

            $overStaffingRedution[$year] = floatval(number_format($this->calcOverStaffingReduction() + $overStaffingRedution[$previousYear], 2, '.', ''));
            $underStaffingRedution[$year] = $this->calcUnderStaffingReduction() + $underStaffingRedution[$previousYear];
            $overTimeRedution[$year] = $this->calcOverTimeReduction() + $overTimeRedution[$previousYear];
            $laborCostImprovement[$year] =      $overStaffingRedution[$year] +
                                                $underStaffingRedution[$year] +
                                                $overTimeRedution[$year];

            $totalSubscriptionCost[$year] = $this->calcTotalSubscriptionCost() * self::MONTHS_PER_YEAR * $year;

            $totalCompany[$year] = $customerRetentionProfits[$year] + $laborCostImprovement[$year] + $totalSubscriptionCost[$year];
        }

        $result = [
            'customerRetentionProfits' => $customerRetentionProfits,
            'overStaffingRedution' => $overStaffingRedution,
            'underStaffingRedution' => $underStaffingRedution,
            'overTimeRedution' => $overTimeRedution,
            'laborCostImprovement' => $laborCostImprovement,
            'totalSubscriptionCost' => $totalSubscriptionCost,
            'totalCompany' => $totalCompany,
        ];

        return $result;
    }
}