<?php
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

require 'Calculator.php';

function sum($carry, $item) {
    return $carry + $item;
}

$calculadora = new Calculator();
if (filter_var($_POST['totalNumberLocations'], FILTER_VALIDATE_INT, array("options" => array("min_range" => 1,"max_range" => 1000)))) {
    $calculadora->setTotalNumberLocations($_POST['totalNumberLocations']);
}

if (filter_var($_POST['teamsPerLocation'], FILTER_VALIDATE_INT, array("options" => array("min_range" => 0,"max_range" => 250)))) {
    $calculadora->setTeamsPerLocation($_POST['teamsPerLocation']);
}

if (filter_var($_POST['employeesPerTeam'], FILTER_VALIDATE_INT, array("options" => array("min_range" => 0,"max_range" => 30)))) {
    $calculadora->setEmployeesPerTeam($_POST['employeesPerTeam']);
}

if (filter_var($_POST['typicalOrdersPerHourLocation'], FILTER_VALIDATE_FLOAT, array("options" => array("min_range" => 0,"max_range" => 200)))) {
    $calculadora->setTypicalOrdersPerHourLocation($_POST['typicalOrdersPerHourLocation']);
}

if (filter_var($_POST['averageProfitPerOrder'], FILTER_VALIDATE_FLOAT, array("options" => array("min_range" => 0,"max_range" => 250)))) {
    $calculadora->setAverageProfitPerOrder($_POST['averageProfitPerOrder']);
}

if (filter_var($_POST['durationBetweenRepeatCustomerVisits'], FILTER_VALIDATE_INT, array("options" => array("min_range" => 1,"max_range" => 52)))) {
    $calculadora->setDurationBetweenRepeatCustomerVisits($_POST['durationBetweenRepeatCustomerVisits']);
}

if (filter_var($_POST['overStaffedCurrent'], FILTER_VALIDATE_INT, array("options" => array("min_range" => 0,"max_range" => 100)))) {   
    $calculadora->setOverStaffedCurrent($_POST['overStaffedCurrent']);
}

if (filter_var($_POST['underStaffedCurrent'], FILTER_VALIDATE_INT, array("options" => array("min_range" => 0,"max_range" => 100)))) {    
    $calculadora->setUnderStaffedCurrent($_POST['underStaffedCurrent']);
}

if (filter_var($_POST['overTimeCurrent'], FILTER_VALIDATE_INT, array("options" => array("min_range" => 0,"max_range" => 250)))) {
    $calculadora->setOverTimeCurrent($_POST['overTimeCurrent']);
}

// echo json_encode($calculadora->calcValuesByWeeks());
// return;
$dataByYear = $calculadora->calcValuesByYear();
$year = 1;
$lastYear = $calculadora->getTotalYears();
$categories = [];
for ($count=0; $count <= $calculadora->getTotalYears() ; $count++) { 
    if ($count > 0) {

        $categories[] = [
            'YR ' . sprintf('%01d', $count)
        ];
    } else {
        $categories[] = [''];
    }
}

$result = [
    "chart_finance" => [
        "categories" => $categories,
        "colors" => ["#0C2A3B", "#2397D0", "#9B9FA7", "#2EBEB9"],
        "series" => [
            [
                "name" => "Customer Retention Profits",
                "data" => $dataByYear['customerRetentionProfits']
            ], 
            [
                "name" => "Labor Cost Improvement",
                "data" => $dataByYear['laborCostImprovement']
            ], 
            // [
            //     "name" => "Software Expense",
            //     "data" => $dataByYear['totalSubscriptionCost']
            // ], 
            [
                "name" => "Total Net Impact",
                "data" => $dataByYear['totalCompany']
            ]
        ]
    ],
    "summary_finance" => [
        "incremental_profit" => [
            "total_company" => [
                "year_1" => $dataByYear['customerRetentionProfits'][$year],
                "year_1_3" => $dataByYear['customerRetentionProfits'][$lastYear]
            ],
            "avg_location" => [
                "year_1" => $dataByYear['customerRetentionProfits'][$year] / $calculadora->getTotalNumberLocations(),
                "year_1_3" => $dataByYear['customerRetentionProfits'][$lastYear] / $calculadora->getTotalNumberLocations(),
            ]
        ],
        "labor_cost_improvement" => [
            "total_company" => [
                "year_1" => $dataByYear['laborCostImprovement'][$year],
                "year_1_3" => $dataByYear['laborCostImprovement'][$lastYear],
            ],
            "avg_location" => [
                "year_1" => $dataByYear['laborCostImprovement'][$year] / $calculadora->getTotalNumberLocations(),
                "year_1_3" => $dataByYear['laborCostImprovement'][$lastYear] / $calculadora->getTotalNumberLocations()
            ]
        ],
        "over_staffing_reduction" => [
            "total_company" => [
                "year_1" => $dataByYear['overStaffingRedution'][$year],
                "year_1_3" => $dataByYear['overStaffingRedution'][$lastYear],
            ],
            "avg_location" => [
                "year_1" => $dataByYear['overStaffingRedution'][$year] / $calculadora->getTotalNumberLocations(),
                "year_1_3" => $dataByYear['overStaffingRedution'][$lastYear] / $calculadora->getTotalNumberLocations()
            ]
        ],
        "under_staffing_reduction" => [
            "total_company" => [
                "year_1" => Calculator::formatNegativeNumber($dataByYear['underStaffingRedution'][$year]),
                "year_1_3" => Calculator::formatNegativeNumber($dataByYear['underStaffingRedution'][$lastYear]),
            ],
            "avg_location" => [
                "year_1" => Calculator::formatNegativeNumber($dataByYear['underStaffingRedution'][$year] / $calculadora->getTotalNumberLocations()),
                "year_1_3" => Calculator::formatNegativeNumber($dataByYear['underStaffingRedution'][$lastYear] / $calculadora->getTotalNumberLocations())
            ]
        ],
        "over_time_reduction" => [
            "total_company" => [
                "year_1" => $dataByYear['overTimeRedution'][$year],
                "year_1_3" => $dataByYear['overTimeRedution'][$lastYear],
            ],
            "avg_location" => [
                "year_1" => $dataByYear['overTimeRedution'][$year] / $calculadora->getTotalNumberLocations(),
                "year_1_3" => $dataByYear['overTimeRedution'][$lastYear] / $calculadora->getTotalNumberLocations()
            ]
        ],
        // "software_expense" => [
        //     "total_company" => [
        //         "year_1" => Calculator::formatNegativeNumber($dataByYear['totalSubscriptionCost'][$year]),
        //         "year_1_3" => Calculator::formatNegativeNumber($dataByYear['totalSubscriptionCost'][$lastYear]),
        //     ],
        //     "avg_location" => [
        //         "year_1" => Calculator::formatNegativeNumber($dataByYear['totalSubscriptionCost'][$year] / $calculadora->getTotalNumberLocations()),
        //         "year_1_3" => Calculator::formatNegativeNumber($dataByYear['totalSubscriptionCost'][$lastYear] / $calculadora->getTotalNumberLocations())
        //     ]
        // ],
        "net_business_impact" => [
            "total_company" => [
                "year_1" => $dataByYear['totalCompany'][$year],
                "year_1_3" => $dataByYear['totalCompany'][$lastYear],
            ],
            "avg_location" => [
                "year_1" => $dataByYear['totalCompany'][$year] / $calculadora->getTotalNumberLocations(),
                "year_1_3" => $dataByYear['totalCompany'][$lastYear] / $calculadora->getTotalNumberLocations()
            ]
        ]
    ]
];
echo json_encode($result);
return;